package de.intevation.flys.importer;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.hibernate.Session;
import org.hibernate.Query;
import org.hibernate.exception.ConstraintViolationException;

import de.intevation.flys.model.River;
import de.intevation.flys.model.Unit;
import de.intevation.flys.model.WaterlevelDifference;


public class ImportWaterlevelDifference {

    private static final Logger log =
        Logger.getLogger(ImportWaterlevelDifference.class);


    private ImportUnit unit;

    private String description;

    private List<ImportWaterlevelDifferenceColumn> columns;

    private WaterlevelDifference peer;


    public ImportWaterlevelDifference(String description) {
        this.columns = new ArrayList<ImportWaterlevelDifferenceColumn>();

        this.description = description;
    }


    public void setUnit(ImportUnit unit) {
        this.unit = unit;
    }


    public void addValue(ImportWaterlevelDifferenceColumn column) {
        this.columns.add(column);
    }


    public void storeDependencies(River river)
    throws SQLException, ConstraintViolationException
    {
        log.info("store dependencies");

        WaterlevelDifference peer = getPeer(river);

        int i = 0;

        for (ImportWaterlevelDifferenceColumn column: columns) {
            column.storeDependencies(peer);
            i++;
        }

        log.info("stored " + i + " waterlevel difference columns");
    }


    public WaterlevelDifference getPeer(River river) {
        Unit u = unit != null ? unit.getPeer() : null;
        if (u == null) {
            log.warn("skip invalid waterlevel difference - no unit set!");
            return null;
        }

        if (peer == null) {
            Session session = ImporterSession.getInstance().getDatabaseSession();
            Query query = session.createQuery(
                "from WaterlevelDifference where " +
                "   river=:river and " +
                "   unit=:unit and " +
                "   description=:description"
            );

            query.setParameter("river", river);
            query.setParameter("unit", u);
            query.setParameter("description", description);

            List<WaterlevelDifference> diffs = query.list();
            if (diffs.isEmpty()) {
                peer = new WaterlevelDifference(river, u, description);
                session.save(peer);
            }
            else {
                peer = diffs.get(0);
            }
        }

        return peer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
